from os import listdir
from os.path import dirname, join
from pathlib import Path

from preql import Environment
from preql.parser import parse


def parse_initial_models(fpath: str) -> Environment:
    files = listdir(dirname(fpath))

    for file in files:
        path = Path(file)
        if file.endswith("entrypoint.preql"):
            with open(join(dirname(fpath), file), "r", encoding="utf-8") as f:
                contents = f.read()
                env = Environment(working_path=dirname(fpath))
                environment, statements = parse(contents, environment=env)
                return environment



import pkgutil
from preql import Environment
from typing import Dict
from os.path import dirname
import sys

models: Dict["str", Environment] = {}

__version__ = "0.0.4"

for loader, module_name, is_pkg in pkgutil.walk_packages([dirname(__file__)]):
    module = loader.find_module(module_name)  # type: ignore
    if not module:
        continue
    _module = module.load_module(module_name)
    try:
        sys.modules["trilogy_public_models." + module_name] = _module.model
        models[module_name] = _module.model
    except AttributeError:
        continue

__all__ = ["models"]



# type: ignore
import ast
import re

import setuptools

_version_re = re.compile(r"__version__\s+=\s+(.*)")
with open("trilogy_public_models/__init__.py", "rb") as f:
    _match = _version_re.search(f.read().decode("utf-8"))
    if _match is None:
        print("No version found")
        raise SystemExit(1)
    version = str(ast.literal_eval(_match.group(1)))


with open("requirements.txt", "r") as f:
    install_requires = [line.strip().replace("==", ">=") for line in f.readlines()]

setuptools.setup(
    name="trilogy-public-models",
    version=version,
    url="",
    author="",
    author_email="preql-community@gmail.com",
    description="Public trilogy/preql models.",
    long_description=open("README.md").read(),
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(exclude=["dist", "build", "*.tests", "*.tests.*", "tests.*", "tests", "docs", ".github", "", "examples"]),
    package_data={
        "": ["*.preql"],
    },
    install_requires=install_requires,
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



from preql import Environment
from preql.core.query_processor import get_datasource_by_concept_and_grain
from preql.core.models import Grain, Concept, Datasource
from preql.executor import Executor
from preql.parser import parse_text


def validate_dataset(
    dataset: Datasource, environment: Environment, executor: Executor, dry_run_client
):
    # TODO: move ths into executor so we don't need this import
    from google.cloud.bigquery import QueryJobConfig

    validation_query = (
        "SELECT\n " + ",\n".join([x.address for x in dataset.concepts]) + " LIMIT 0;"
    )

    try:
        _, parsed = parse_text(validation_query, environment)
        sql = executor.generator.generate_queries(environment, parsed)
    except Exception as e:
        print(validation_query)
        raise e
    for statement in sql:
        # Start the query, passing in the extra configuration.
        try:
            # for UI execution, cap the limit
            compiled_sql = executor.generator.compile_statement(statement)

            # TODO: implement this
            # rs = executor.engine.dry_run(compiled_sql)
            job_config = QueryJobConfig(dry_run=True, use_query_cache=False)
            query_job = dry_run_client.query(
                compiled_sql, job_config=job_config
            )  # Make an API request.
        except Exception as e:
            print("Failed validation on:")
            print(validation_query)
            print(compiled_sql)
            raise e

        # TODO: do something interesting with cost modeling?
        #
        print(
            "This query will process {} bytes.".format(query_job.total_bytes_processed)
        )


def validate_concept(concept: Concept, env):
    get_datasource_by_concept_and_grain(
        concept, grain=Grain(components=[concept.with_default_grain()]), environment=env
    )


def validate_model(model: Environment, executor: Executor, dry_run_client):
    for dataset in model.datasources.values():
        validate_dataset(dataset, model, executor, dry_run_client)
    for concept in model.concepts.values():
        validate_concept(concept, model)
